/*********************************************************************************
  *Copyright(C),2015-2020, 
  *TUYA 
  *www.tuya.comm
  *FileName:    tuya_ipc_dp_utils.h
  *
  *Do not modify any of the contents of this file at will.
  Please contact the Product Manager if you need to modify it! !
  * File Description：
  * 1. DP point definition
  * 2. DP point API definition
**********************************************************************************/

#ifndef _TY_DP_DEFINE_H
#define _TY_DP_DEFINE_H

#ifdef __cplusplus
extern "C" {
#endif

#include "tuya_cloud_types.h"
#include "tuya_ipc_api.h"
#include "tuya_cloud_com_defs.h"

#include "cJSON.h"

#define TUYA_DP_LED 101
#define TUYA_DP_SUPPORT_STORAGE 102
#define TUYA_DP_STOR_VOLUME 103
#define TUYA_DP_STOR_STATUS 104
#define TUYA_DP_STOR_FORMAT 105
#define TUYA_DP_STOR_FORMAT_STATUS 107
#define TUYA_DP_SET_CUR_STORAGE 113
#define TUYA_DP_SIREN_SWITCH 114
#define TUYA_DP_WAKEUP_SUB_DEV 149
#define TUYA_DP_ELECTRICITY 145
#define TUYA_DP_POWERMODE 146
#define TUYA_DP_SET_SUB_SLEEPTIME 235

/* Report the latest status of all local DP points*/
VOID IPC_APP_upload_all_status(VOID);

#ifdef TUYA_DP_SD_FORMAT_STATUS_ONLY_GET
/* SD card formatting progress report*/
VOID IPC_APP_report_sd_format_status(INT_T status);
#endif

#ifdef TUYA_DP_SD_STATUS_ONLY_GET
/*When the SD card changes (plug and unplug), call this API to notify to APP*/
VOID IPC_APP_report_sd_status_changed(INT_T status);
#endif

#ifdef TUYA_DP_SD_STORAGE_ONLY_GET
/* After formatting, call this API to report the storage capacity. */
VOID IPC_APP_report_sd_storage();
#endif


#ifdef TUYA_DP_POWERMODE
/* Call this API to notify the app when the battery status changes. */
VOID IPC_APP_update_battery_status(VOID);
#endif

#ifdef TUYA_DP_MOTION_DETECTION_ALARM
/* When a motion detection event occurs, the API is called to push an alarm picture to the APP. */
OPERATE_RET IPC_APP_Send_Motion_Alarm_From_Buffer(CHAR_T *data, UINT_T size, NOTIFICATION_CONTENT_TYPE_E type);
OPERATE_RET IPC_APP_Send_Motion_Alarm(CHAR_T *p_abs_file, NOTIFICATION_CONTENT_TYPE_E file_type);
#endif

#ifdef TUYA_DP_DOOR_BELL_SNAP
/* When the doorbell is pressed, it will captures a picture and calls the API to push the picture to the APP. */
OPERATE_RET IPC_APP_Send_DoorBell_Snap(CHAR_T *p_snap_file, NOTIFICATION_CONTENT_TYPE_E file_type);
#endif

/* basic API */
VOID IPC_APP_handle_dp_cmd_objs(IN CONST TY_RECV_OBJ_DP_S *dp_rev);
/* API */
VOID IPC_APP_handle_dp_query_objs(IN CONST TY_DP_QUERY_S *dp_query);
VOID IPC_APP_handle_raw_dp_cmd_objs(IN CONST TY_RECV_RAW_DP_S *dp_rev);

#if defined(TUYA_EXTEND_DEVELOP)
OPERATE_RET APP_DP_VALUE_CMD_PROC(IN CONST INT_T dp_id, IN CONST INT_T dp_value,OUT  INT_T * dp_report_value);
OPERATE_RET APP_DP_BOOL_CMD_PROC(IN CONST INT_T dp_id, IN CONST BOOL_T dp_value,OUT  BOOL_T * dp_report_value );
OPERATE_RET APP_DP_STR_CMD_PROC(IN CONST INT_T dp_id, IN CONST CHAR_T *dp_value,OUT  CHAR_T * dp_report_value);
OPERATE_RET APP_DP_ENUM_CMD_PROC(IN CONST INT_T dp_id, IN CONST UINT_T dp_value,OUT  CHAR_T * dp_report_value);
OPERATE_RET APP_DP_BITMAP_CMD_PROC(IN CONST INT_T dp_id, IN CONST UINT_T dp_value,OUT  UINT_T * dp_report_value);
#endif

INT_T IPC_APP_get_ap_mode_config();

INT_T IPC_APP_switch_ap_mode();

#ifdef __cplusplus
}
#endif

#endif  /*_TY_DP_DEFINE_H*/
