#include <libgen.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/prctl.h>
#include <unistd.h>

#include "tal_system.h"
#include "tkl_audio.h"
#include "tuya_iot_base_api.h"
#include "tuya_iot_config.h"
#include "tuya_ipc_sdk_init.h"
#include "tuya_svc_netmgr_linkage.h"
#include "ty_sdk_call.h"
#include "ty_sdk_common.h"
#include "utilities/uni_log.h"

#include "tuya_cloud_base_defs.h"
#include "tuya_gw_subdev_api.h"
#include "tuya_xvr_dev.h"

#define LAN_AP_SSID_PREFIX "SmartLife-"
#define LAN_AP_SSID_SUFFIX_LEN 4
#define WIFI_SSID_LEN 32

#define DEMO_ENABLE_TUYA_AUDIO_INTERACTION 0

#define USER_DEV_DTL_IPC (0x2 << 24)

STATIC INT_T s_mode = -1;
STATIC CHAR_T s_token[30] = { 0 };
STATIC BOOL_T s_is_lan_only_mode = 0;

CHAR_T s_raw_path[128] = "/tmp/";

CHAR_T s_ipc_pid[64] = "tuya_pid";         // Product ID of TUYA device, this is for demo only.
CHAR_T s_ipc_uuid[64] = "tuya_uuid";       // Unique identification of each device//Contact tuya PM/BD for developing devices or BUY more
CHAR_T s_ipc_authkey[64] = "tuya_authkey"; // Authentication codes corresponding to UUID, one machine one code, paired with UUID.
CHAR_T s_ipc_storage[64] = "/tmp/";        // Path to save tuya sdk DB files, should be readable, writeable and storable
CHAR_T s_ipc_env[64] = "";
CHAR_T s_ipc_sd_path[128] = "/tmp/";                  // SD card mount directory
CHAR_T s_ipc_upgrade_file[128] = "/tmp/upgrade.file"; // File with path to download file during OTA
CHAR_T s_app_version[64] = "1.2.3";                   // Firemware version displayed on TUYA APP


STATIC INT_T sg_sub_enable[MAX_DEV_NUM] = {0};
STATIC CHAR_T sg_nvr_sub_pid[MAX_DEV_NUM][64] = {0};
STATIC CHAR_T sg_sub_ipc_uuid[MAX_DEV_NUM][64] = {0};
STATIC CHAR_T sg_sub_ipc_authkey[MAX_DEV_NUM][64] = {0};

STATIC TUYA_IPC_SDK_RUN_VAR_S g_sdk_run_info = { 0 };

extern VOID TUYA_IPC_Status_Changed_cb(IN TUYA_IPC_STATUS_GROUP_E changed_group, IN CONST TUYA_IPC_STATUS_E status[TUYA_IPC_STATUS_GROUP_MAX]);
extern INT_T TUYA_IPC_sd_status_upload(INT_T status);
extern VOID TUYA_IPC_qrcode_shorturl_cb(CHAR_T* shorturl);
extern INT_T TUYA_IPC_Upgrade_Inform_cb(IN CONST FW_UG_S* fw);
extern VOID TUYA_IPC_Reset_System_CB(GW_RESET_TYPE_E type);
extern VOID TUYA_IPC_Restart_Process_CB(VOID);
extern INT_T TUYA_IPC_Get_MqttStatus();

extern INT_T TUYA_IPC_p2p_event_cb(IN CONST INT_T device, IN CONST INT_T channel, IN CONST MEDIA_STREAM_EVENT_E event, IN PVOID_T args);
extern VOID TUYA_IPC_APP_rev_audio_cb(IN INT_T device, IN INT_T channel, IN CONST MEDIA_AUDIO_FRAME_T* p_audio_frame);
extern VOID TUYA_IPC_APP_rev_video_cb(IN INT_T device, IN INT_T channel, IN CONST MEDIA_VIDEO_FRAME_T* p_video_frame);
extern VOID TUYA_IPC_APP_rev_file_cb(IN INT_T device, IN INT_T channel, IN CONST MEDIA_FILE_DATA_T* p_file_data);
extern VOID TUYA_APP_get_snapshot_cb(IN INT_T device, IN INT_T channel, OUT CHAR_T* snap_addr, OUT INT_T* snap_size);
extern VOID TUYA_IPC_Media_Adapter_Init(TUYA_IPC_SDK_MEDIA_ADAPTER_S* p_media_adatper_info, TUYA_IPC_SDK_MEDIA_STREAM_S* p_media_infos);
extern VOID TUYA_IPC_Media_Stream_Init(TUYA_IPC_SDK_MEDIA_ADAPTER_S* p_media_adatper_info);
extern OPERATE_RET TUYA_APP_Init_Ring_Buffer(CONST IPC_MEDIA_INFO_T* pMediaInfo, INT_T channel);
extern OPERATE_RET TUYA_APP_Put_Frame(RING_BUFFER_USER_HANDLE_T handle, IN CONST MEDIA_FRAME_T* p_frame);

extern VOID TUYA_IPC_upload_all_status(VOID);
extern VOID TUYA_IPC_handle_dp_cmd_objs(IN CONST TY_RECV_OBJ_DP_S* dp_rev);
extern VOID TUYA_IPC_handle_raw_dp_cmd_objs(IN CONST TY_RECV_RAW_DP_S* dp_rev);
extern VOID TUYA_IPC_handle_dp_query_objs(IN CONST TY_DP_QUERY_S* dp_query);
extern VOID TUYA_IPC_INIT_DP_CONFIG();

BOOL_T tuya_get_lan_mode()
{
    return s_is_lan_only_mode;
}

VOID_T tuya_set_lan_mode(BOOL_T on_off)
{
    s_is_lan_only_mode = on_off;
    return;
}

void signal_handle(int sig)
{
    char cThreadName[32] = { 0 };
    prctl(PR_GET_NAME, (unsigned long)cThreadName);
    printf("[%s, %d] get signal(%d)  name(%s)\n", __FUNCTION__, __LINE__, sig, cThreadName);

    switch (sig) {
    case SIGINT:
    case SIGKILL:
    case SIGTERM:
    case SIGSEGV:
    case SIGABRT:
        exit(1);
        break;

    default:
        break;
    }

    return;
}

static void __USAGE(char* app_name)
{
    printf("%s -m mode -t token -r raw path -h\n", (char*)basename(app_name));
    printf("\t m:  1-PARING_MODE_WIFI_AP 2-PARING_MODE_DEBUG 3-PARING_MODE_WIRED, refer to TUYA_IPC_PARING_MODE_E\n"
           "\t t: token get form qrcode info\n"
           "\t r: raw source file path\n"
           "\t h: help info\n");

    return;
}

static int __TUYA_IPC_DEMO_CFG_PARSE(char* cfg_path)
{
    char buffer[4096];
    int read_len = sizeof(buffer);
    int fd = -1;
    ty_cJSON* cfg_json = NULL;

    if (cfg_path == NULL) {
        printf("cfg_path is null\n");
        return -1;
    }

    // 从配置路径里读取相关的配置文件，整体的配置环境需要守护程序建立
    fd = open(cfg_path, 2);
    printf("detect auto test enviroment, load config from %s\n", cfg_path);
    // 检查是打开文件成功。
    if (fd < 0) {
        printf("open path %s failed,%d\n", cfg_path, fd);
        return -1;
    }

    // 读取文件内容
    memset(buffer, 0, sizeof(buffer));
    while (read_len == read(fd, buffer, sizeof(buffer)))
        ;
    close(fd);
    printf("buffer is %s\n", buffer);

    // 解析JSON
    cfg_json = ty_cJSON_Parse(buffer);
    if (NULL == cfg_json) {
        printf("load configure failed\n");
        return OPRT_CJSON_PARSE_ERR;
    }
    ty_cJSON* env = ty_cJSON_GetObjectItem(cfg_json, "env");
    if (env) {
        strcpy(s_ipc_env, env->valuestring);
        printf("set env to %s\n", s_ipc_env);
        tuya_svc_mqtt_direct_set_env(s_ipc_env);
    }
    // pid
    ty_cJSON* json_pid = ty_cJSON_GetObjectItem(cfg_json, "pid");
    if (!ty_cJSON_IsString(json_pid)) {
        printf("get pid failed\n");
        ty_cJSON_Delete(cfg_json);
        return -1;
    }
    strcpy(s_ipc_pid, json_pid->valuestring);
    // uuid
    ty_cJSON* json_uuid = ty_cJSON_GetObjectItem(cfg_json, "uuid");
    if (!ty_cJSON_IsString(json_uuid)) {
        printf("get uuid failed\n");
        ty_cJSON_Delete(cfg_json);
        return -1;
    }
    strcpy(s_ipc_uuid, json_uuid->valuestring);
    // authkey
    ty_cJSON* json_authkey = ty_cJSON_GetObjectItem(cfg_json, "authkey");
    if (!ty_cJSON_IsString(json_authkey)) {
        printf("get authkey failed\n");
        ty_cJSON_Delete(cfg_json);
        return -1;
    }
    strcpy(s_ipc_authkey, json_authkey->valuestring);
    // storage path
    ty_cJSON* json_storage = ty_cJSON_GetObjectItem(cfg_json, "storage");
    if (ty_cJSON_IsString(json_storage)) {
        strcpy(s_ipc_storage, json_storage->valuestring);
    }
    // media path
    ty_cJSON* json_media = ty_cJSON_GetObjectItem(cfg_json, "media_path");
    if (ty_cJSON_IsString(json_media)) {
        strcpy(s_raw_path, json_media->valuestring);
    }
    // sd path
    ty_cJSON* json_sd_path = ty_cJSON_GetObjectItem(cfg_json, "sd_path");
    if (ty_cJSON_IsString(json_sd_path)) {
        strcpy(s_ipc_sd_path, json_sd_path->valuestring);
    }

    // upgrade file
    ty_cJSON* json_upgrade_file = ty_cJSON_GetObjectItem(cfg_json, "upgrade_file");
    if (ty_cJSON_IsString(json_upgrade_file)) {
        strcpy(s_ipc_upgrade_file, json_upgrade_file->valuestring);
    }
    // app version
    ty_cJSON* json_app_version = ty_cJSON_GetObjectItem(cfg_json, "app_version");
    if (ty_cJSON_IsString(json_app_version)) {
        strcpy(s_app_version, json_app_version->valuestring);
    }

    // token
    ty_cJSON* json_token = ty_cJSON_GetObjectItem(cfg_json, "token");
    if (ty_cJSON_IsString(json_token)) {
        strcpy(s_token, json_token->valuestring);
    }
    // mode
    ty_cJSON* json_mode = ty_cJSON_GetObjectItem(cfg_json, "mode");
    if (ty_cJSON_IsString(json_mode)) {
        s_mode = atoi(json_mode->valuestring);
    }
    ty_cJSON_Delete(cfg_json);

    printf("s_ipc_pid: %s\n", s_ipc_pid);
    printf("s_ipc_uuid: %s\n", s_ipc_uuid);
    printf("s_ipc_authkey: %s\n", s_ipc_authkey);
    printf("s_ipc_storage: %s\n", s_ipc_storage);
    printf("s_raw_path: %s\n", s_raw_path);
    printf("s_ipc_sd_path: %s\n", s_ipc_sd_path);
    printf("s_ipc_upgrade_file: %s\n", s_ipc_upgrade_file);
    printf("s_app_version: %s\n", s_app_version);

    return 0;
}

VOID TUYA_IPC_simulation()
{
    /* Manual simulation of related functions */
    char test_input[64] = { 0 };
    int ret = 0;
    while (1) {
        scanf("%s", test_input);
        if (0 == strcmp(test_input, "status")) {
            IPC_REGISTER_STATUS_E status = tuya_ipc_get_register_status();
            PR_INFO("current register status %d[0:unregistered 1:registered 2:activated]\n", status);
        }
        /* Simulation of doorbell press event */
        else if (0 == strcmp(test_input, "bell_press")) {
            // Using demo file for simulation, should be replaced by real snapshot when events happen.
            int snapshot_size = 150 * 1024;
            char* snapshot_buf = (char*)malloc(snapshot_size);
            int ret = IPC_APP_get_snapshot(snapshot_buf, &snapshot_size);
            if (ret != 0) {
                PR_ERR("Get snap fail!\n");
                free(snapshot_buf);
                continue;
            }
            /* Push the detection message and the current snapshot image to the APP.
            Snapshot image acquisition needs to be implemented by the developer */
            tuya_ipc_door_bell_press(DOORBELL_NORMAL, snapshot_buf, snapshot_size, NOTIFICATION_CONTENT_JPEG);
            free(snapshot_buf);
        }
        /* Simulation of get time for OSD */
        else if (0 == strcmp(test_input, "osd")) {
            struct tm localTime;
            tuya_ipc_get_tm_with_timezone_dls(&localTime);
            PR_DEBUG("show OSD [%04d-%02d-%02d %02d:%02d:%02d]", localTime.tm_year, localTime.tm_mon, localTime.tm_mday, localTime.tm_hour, localTime.tm_min, localTime.tm_sec);
        }
#if defined(ENABLE_TUYA_CALL) && (ENABLE_TUYA_CALL == 1)
        else if (0 == strcmp(test_input, "call")) {
            TUYA_IPC_call_app();
        } else if (0 == strcmp(test_input, "hangup")) {
            TUYA_IPC_hangup();
        } else if (0 == strcmp(test_input, "download")) {
            TUYA_IPC_download_background_demo();
        }
#endif
        else if (0 == strcmp(test_input, "start_rtsp")) {
            start_rtsp_client();
        } else if (0 == strcmp(test_input, "stop_rtsp")) {
            stop_rtsp_client();
        } else if (0 == strcmp(test_input, "add_sub_device")) {
            TUYA_APP_Add_Xvr_Sub_Dev();
        } else if (0 == strcmp(test_input, "get_sub_device")) {
            TUYA_APP_Get_Xvr_Sub_Dev();
        } else if (0 == strcmp(test_input, "remove_sub_device")) {
            TUYA_APP_Remove_Xvr_Sub_Dev();
        } else if (0 == strcmp(test_input, "start_ev")) {
            tuya_ipc_ss_start_event_by_type("", 1);
        } else if (0 == strcmp(test_input, "stop_ev")) {
            tuya_ipc_ss_stop_event_by_type("", 1);
        } else if (0 == strcmp(test_input, "set_storage_none_mode")) {
            tuya_ipc_ss_set_write_mode(SS_WRITE_MODE_NONE);
        } else if (0 == strcmp(test_input, "set_storage_all_mode")) {
            tuya_ipc_ss_set_write_mode(SS_WRITE_MODE_ALL);
        } else if (0 == strcmp(test_input, "set_storage_event_mode")) {
            tuya_ipc_ss_set_write_mode(SS_WRITE_MODE_EVENT);
        } else if (NULL != strstr(test_input, "start_md_subdev_")) { //simulated sub device motion detection.
            int sub_num;
            if (sscanf(test_input, "start_md_subdev_%d", &sub_num) == 1) {
                if(sub_num > MAX_DEV_NUM || sub_num < 0) {
                    PR_ERR("sub num[%d] is illegal",sub_num);
                } else {
                    IPC_APP_set_motion_status(1, sg_sub_ipc_uuid[sub_num]);
                }
            }
         } else if (NULL != strstr(test_input, "stop_md_subdev_")) {
            int sub_num;
            if (sscanf(test_input, "stop_md_subdev_%d", &sub_num) == 1) {
                if(sub_num > MAX_DEV_NUM || sub_num < 0) {
                    PR_ERR("sub num[%d] is illegal",sub_num);
                } else {
                    IPC_APP_set_motion_status(0, sg_sub_ipc_uuid[sub_num]);
                }
            }
         }
        tal_system_sleep(100);
    }

    return;
}

static OPERATE_RET tuya_set_lan_ap_ssid(CHAR_T* uuid)
{
    if (!uuid) {
        PR_ERR("uuid is null");
        return -1;
    }
    INT_T uuid_len = strlen(uuid);
    if (uuid_len <= LAN_AP_SSID_SUFFIX_LEN) {
        PR_ERR("uuid len %d is illegal", uuid_len);
        return -1;
    }

    CHAR_T ap_ssid[WIFI_SSID_LEN + 1] = { 0 };
    snprintf(ap_ssid, sizeof(ap_ssid) - 1, "%s%s", LAN_AP_SSID_PREFIX, uuid + uuid_len - LAN_AP_SSID_SUFFIX_LEN);
    tuya_iot_set_user_def_ap_if(ap_ssid, NULL);
    return OPRT_OK;
}

VOID TUYA_APP_Get_Media_Info(IN CONST INT_T chn, IN OUT IPC_MEDIA_INFO_T* param)
{
    // demo 所展示设备的信息是一样的。故没有区分设备。用户根据chn区分不同设备多媒体信息
    //确保媒体信息和IPC_APP_Set_Media_Info函数设置是一致的
    param->stream_enable[E_IPC_STREAM_VIDEO_MAIN] = TRUE;                  /* 是否开启本地高清视频流 */
    param->video_fps[E_IPC_STREAM_VIDEO_MAIN] = 30;                        /* FPS */
    param->video_gop[E_IPC_STREAM_VIDEO_MAIN] = 30;                        /* GOP */
    param->video_bitrate[E_IPC_STREAM_VIDEO_MAIN] = TUYA_VIDEO_BITRATE_2M; /* 码率上限 */
    param->video_width[E_IPC_STREAM_VIDEO_MAIN] = 640;                     /* 单帧分辨率 宽 */
    param->video_height[E_IPC_STREAM_VIDEO_MAIN] = 360;                    /* 单帧分辨率 高 */
    param->video_freq[E_IPC_STREAM_VIDEO_MAIN] = 90000;                    /* 时钟频率 */
    param->video_codec[E_IPC_STREAM_VIDEO_MAIN] = TUYA_CODEC_VIDEO_H264;   /* 编码格式 */

    //只需要高清和音频
    /* 请注意如果设备子码流支持多种视频流配置，这里请把各项设置为允许配置的上限值 */
    param->stream_enable[E_IPC_STREAM_VIDEO_SUB] = TRUE;                  /* 是否开启本地标清视频流 */
    param->video_fps[E_IPC_STREAM_VIDEO_SUB] = 30;                        /* FPS */
    param->video_gop[E_IPC_STREAM_VIDEO_SUB] = 30;                        /* GOP */
    param->video_bitrate[E_IPC_STREAM_VIDEO_SUB] = TUYA_VIDEO_BITRATE_2M; /* 码率上限 */
    param->video_width[E_IPC_STREAM_VIDEO_SUB] = 640;                     /* 单帧分辨率 宽 */
    param->video_height[E_IPC_STREAM_VIDEO_SUB] = 360;                    /* 单帧分辨率 高 */
    param->video_freq[E_IPC_STREAM_VIDEO_SUB] = 90000;                    /* 时钟频率 */
    param->video_codec[E_IPC_STREAM_VIDEO_SUB] = TUYA_CODEC_VIDEO_H264;   /* 编码格式 */

    /* 音频码流配置 */
    param->stream_enable[E_IPC_STREAM_AUDIO_MAIN] = TRUE;                    /* 是否开启本地声音采集 */
    param->audio_codec[E_IPC_STREAM_AUDIO_MAIN] = TUYA_CODEC_AUDIO_PCM;      /* 编码格式 */
    param->audio_sample[E_IPC_STREAM_AUDIO_MAIN] = TUYA_AUDIO_SAMPLE_8K;     /* 采样率 */
    param->audio_databits[E_IPC_STREAM_AUDIO_MAIN] = TUYA_AUDIO_DATABITS_16; /* 位宽 */
    param->audio_channel[E_IPC_STREAM_AUDIO_MAIN] = TUYA_AUDIO_CHANNEL_MONO; /* 信道 */
    param->audio_fps[E_IPC_STREAM_AUDIO_MAIN] = 25;                          /* 每秒分片数 */

    param->stream_enable[E_IPC_STREAM_AUDIO_SUB] = TRUE;                    /* 是否开启本地声音采集 */
    param->audio_codec[E_IPC_STREAM_AUDIO_SUB] = TUYA_CODEC_AUDIO_AAC_ADTS; /* 编码格式 */
    param->audio_sample[E_IPC_STREAM_AUDIO_SUB] = TUYA_AUDIO_SAMPLE_8K;     /* 采样率 */
    param->audio_databits[E_IPC_STREAM_AUDIO_SUB] = TUYA_AUDIO_DATABITS_16; /* 位宽 */
    param->audio_channel[E_IPC_STREAM_AUDIO_SUB] = TUYA_AUDIO_CHANNEL_MONO; /* 信道 */
    param->audio_fps[E_IPC_STREAM_AUDIO_SUB] = 8;                           /* 每秒分片数 */
}

OPERATE_RET TUYA_APP_Init_All_Dev_Ring_Buffer()
{
    OPERATE_RET ret = OPRT_OK;
    INT_T i = 0;
    INT_T bind_cnt = 0;
    INT_T chanArr[SUB_DEV_MAX_NUM] = { 0 };
    if (0 == tuya_xvr_dev_get_all_sub_nodes_rinfbuf_idx(&bind_cnt, (INT_T*)&chanArr)) {
        // step:循环初始化每一路设备的ringbuffer通道
        for (i = 0; i < bind_cnt; i++) {
            TUYA_XVR_DEV_INFO_T sub_dev_info = { 0 };
            memset(&sub_dev_info, 0, sizeof(sub_dev_info));
            tuya_xvr_dev_get_sub_dev_info_by_ringbuf_idx(chanArr[i], &sub_dev_info);

            IPC_MEDIA_INFO_T media_info = { 0 };
            TUYA_APP_Get_Media_Info(sub_dev_info.ringbuf_idx, &media_info);

            DEVICE_MEDIA_INFO_T device_media_info = { 0 };
            memcpy(&device_media_info.av_encode_info, &media_info, sizeof(IPC_MEDIA_INFO_T));
            device_media_info.audio_decode_info.enable = 1;
            device_media_info.audio_decode_info.audio_codec = TUYA_CODEC_AUDIO_G711U;
            device_media_info.audio_decode_info.audio_sample = TUYA_AUDIO_SAMPLE_8K;
            device_media_info.audio_decode_info.audio_databits = TUYA_AUDIO_DATABITS_16;
            device_media_info.audio_decode_info.audio_channel = TUYA_AUDIO_CHANNEL_MONO;
            device_media_info.max_pic_len = 100 * 1024;

            tuya_ipc_media_adapter_set_media_info(sub_dev_info.ringbuf_idx, 0, device_media_info);

            PR_DEBUG("sub dev id[%s] rinfbuf_idx[%d] ",sub_dev_info.sub_node_id, sub_dev_info.ringbuf_idx);
            TUYA_APP_Init_Ring_Buffer(&media_info, sub_dev_info.ringbuf_idx);

            TUYA_APP_set_dev_enable(sub_dev_info.ringbuf_idx);
        }
    }
    return ret;
}

static OPERATE_RET tuya_ipc_app_start(IN TUYA_IPC_SDK_RUN_VAR_S* pRunInfo)
{
    if (NULL == pRunInfo) {
        PR_ERR("start sdk para is NULL\n");
        return OPRT_INVALID_PARM;
    }

    OPERATE_RET ret = 0;
    STATIC BOOL_T s_ipc_sdk_started = FALSE;
    if (TRUE == s_ipc_sdk_started) {
        PR_DEBUG("IPC SDK has started\n");
        return ret;
    }

    memcpy(&g_sdk_run_info, pRunInfo, SIZEOF(TUYA_IPC_SDK_RUN_VAR_S));

    // setup1:init sdk
    TUYA_IPC_ENV_VAR_T env;
    memset(&env, 0, sizeof(TUYA_IPC_ENV_VAR_T));
    strcpy(env.storage_path, pRunInfo->iot_info.cfg_storage_path);
    strcpy(env.product_key, pRunInfo->iot_info.product_key);
    strcpy(env.uuid, pRunInfo->iot_info.uuid);
    strcpy(env.auth_key, pRunInfo->iot_info.auth_key);
    strcpy(env.dev_sw_version, pRunInfo->iot_info.dev_sw_version);
    strcpy(env.dev_serial_num, "tuya_ipc");
    // TODO:raw
    env.dev_raw_dp_cb = pRunInfo->dp_info.raw_dp_cmd_proc;
    env.dev_obj_dp_cb = pRunInfo->dp_info.common_dp_cmd_proc;
    env.dev_dp_query_cb = pRunInfo->dp_info.dp_query;
    env.status_changed_cb = pRunInfo->net_info.ipc_status_change_cb;
    env.upgrade_cb_info.upgrade_cb = pRunInfo->upgrade_info.upgrade_cb;
    env.gw_rst_cb = pRunInfo->iot_info.gw_reset_cb;
    env.gw_restart_cb = pRunInfo->iot_info.gw_restart_cb;
    env.qrcode_active_cb = pRunInfo->qrcode_active_cb;
    env.dev_type = pRunInfo->iot_info.dev_type;
    env.link_type = pRunInfo->net_info.link_type;
    env.ip_mode_type = pRunInfo->net_info.ip_mode_type;
    env.lan_only_mode = tuya_get_lan_mode();
    ret = tuya_ipc_init_sdk(&env);
    if (OPRT_OK != ret) {
        PR_ERR("init sdk is error\n");
        return ret;
    }

    //设置日志等级
    tuya_ipc_set_log_attr(pRunInfo->debug_info.log_level, NULL);

    // setup2: set media adapter
    TUYA_IPC_Media_Adapter_Init(&pRunInfo->media_adatper_info, &pRunInfo->media_info);

    // 在添加子设备后初始化对应的ringbuf

    tuya_ipc_ring_buffer_adapter_register_media_source();

    if (tuya_get_lan_mode() == 1) {
        // For lan only mode, the AP name should be "SmartLife-" followed by the last four digits of the UUID.
        ret = tuya_set_lan_ap_ssid(env.uuid);
        if (OPRT_OK != ret) {
            PR_ERR("set lan ap ssid fail");
            return ret;
        }
    }

    ret = tuya_ipc_start_sdk(pRunInfo->net_info.connect_mode, pRunInfo->debug_info.qrcode_token);
    if (OPRT_OK != ret) {
        PR_ERR("start sdk is error\n");
        return ret;
    }

    tuya_xvr_dev_init();
    /* step:init ring buffer for user to append av frame */
    TUYA_APP_Init_All_Dev_Ring_Buffer();

    s_ipc_sdk_started = true;
    PR_DEBUG("tuya ipc sdk start is complete\n");
    return ret;
}

static OPERATE_RET __TUYA_IPC_SDK_START(TUYA_IPC_PARING_MODE_E connect_mode, CHAR_T* p_token)
{
    printf("SDK Version:%s\r\n", tuya_ipc_get_sdk_info());
    TUYA_IPC_LINK_TYPE_E link_type = TUYA_IPC_LINK_WIFI;
    TUYA_IPC_SDK_RUN_VAR_S ipc_sdk_run_var = { 0 };
    memset(&ipc_sdk_run_var, 0, sizeof(ipc_sdk_run_var));

    /*certification information(essential)*/
    strcpy(ipc_sdk_run_var.iot_info.product_key, s_ipc_pid);
    strcpy(ipc_sdk_run_var.iot_info.uuid, s_ipc_uuid);
    strcpy(ipc_sdk_run_var.iot_info.auth_key, s_ipc_authkey);
    strcpy(ipc_sdk_run_var.iot_info.dev_sw_version, s_app_version);
    strcpy(ipc_sdk_run_var.iot_info.cfg_storage_path, s_ipc_storage);
    // normal device
    ipc_sdk_run_var.iot_info.dev_type = NORMAL_POWER_DEV;
    // if needed, change to low power device
    // ipc_sdk_run_var.iot_info.dev_type= LOW_POWER_DEV;

    /*connect mode (essential)*/
    ipc_sdk_run_var.net_info.connect_mode = connect_mode;
    ipc_sdk_run_var.net_info.ipc_status_change_cb = TUYA_IPC_Status_Changed_cb;
    ipc_sdk_run_var.net_info.link_type = link_type;
    ipc_sdk_run_var.net_info.ip_mode_type = TUYA_IPC_IPV4_ONLY;
    printf("MODE:%d  LINK_TYPE:%d IP_MODE_TYPE:%d\r\n", connect_mode, ipc_sdk_run_var.net_info.link_type, ipc_sdk_run_var.net_info.ip_mode_type);
    if (p_token) {
        strcpy(ipc_sdk_run_var.debug_info.qrcode_token, p_token);
    }
    /* 0-5, the bigger, the more log */
    ipc_sdk_run_var.debug_info.log_level = 4;
    /*media info (essential)*/
    /* main stream(HD), video configuration*/
    /* NOTE
    FIRST:If the main stream supports multiple video stream configurations, set each item to the upper limit of the allowed configuration.
    SECOND:E_IPC_STREAM_VIDEO_MAIN must exist.It is the data source of SDK.
    please close the E_IPC_STREAM_VIDEO_SUB for only one stream*/
    ipc_sdk_run_var.media_info.media_info.stream_enable[E_IPC_STREAM_VIDEO_MAIN] = TRUE;                  /* Whether to enable local HD video streaming */
    ipc_sdk_run_var.media_info.media_info.video_fps[E_IPC_STREAM_VIDEO_MAIN] = 30;                        /* FPS */
    ipc_sdk_run_var.media_info.media_info.video_gop[E_IPC_STREAM_VIDEO_MAIN] = 30;                        /* GOP */
    ipc_sdk_run_var.media_info.media_info.video_bitrate[E_IPC_STREAM_VIDEO_MAIN] = TUYA_VIDEO_BITRATE_2M; /* Rate limit */
    ipc_sdk_run_var.media_info.media_info.video_width[E_IPC_STREAM_VIDEO_MAIN] = 640;                     /* Single frame resolution of width*/
    ipc_sdk_run_var.media_info.media_info.video_height[E_IPC_STREAM_VIDEO_MAIN] = 360;                    /* Single frame resolution of height */
    ipc_sdk_run_var.media_info.media_info.video_freq[E_IPC_STREAM_VIDEO_MAIN] = 90000;                    /* Clock frequency */
    ipc_sdk_run_var.media_info.media_info.video_codec[E_IPC_STREAM_VIDEO_MAIN] = TUYA_CODEC_VIDEO_H264;   /* Encoding format */

    /* substream(HD), video configuration */
    /* Please note that if the substream supports multiple video stream configurations, please set each item to the upper limit of the allowed configuration. */
    ipc_sdk_run_var.media_info.media_info.stream_enable[E_IPC_STREAM_VIDEO_SUB] = TRUE;                    /* Whether to enable local SD video stream */
    ipc_sdk_run_var.media_info.media_info.video_fps[E_IPC_STREAM_VIDEO_SUB] = 30;                          /* FPS */
    ipc_sdk_run_var.media_info.media_info.video_gop[E_IPC_STREAM_VIDEO_SUB] = 30;                          /* GOP */
    ipc_sdk_run_var.media_info.media_info.video_bitrate[E_IPC_STREAM_VIDEO_SUB] = TUYA_VIDEO_BITRATE_512K; /* Rate limit */
    ipc_sdk_run_var.media_info.media_info.video_width[E_IPC_STREAM_VIDEO_SUB] = 640;                       /* Single frame resolution of width */
    ipc_sdk_run_var.media_info.media_info.video_height[E_IPC_STREAM_VIDEO_SUB] = 360;                      /* Single frame resolution of height */
    ipc_sdk_run_var.media_info.media_info.video_freq[E_IPC_STREAM_VIDEO_SUB] = 90000;                      /* Clock frequency */
    ipc_sdk_run_var.media_info.media_info.video_codec[E_IPC_STREAM_VIDEO_SUB] = TUYA_CODEC_VIDEO_H264;     /* Encoding format */

    /* Audio stream configuration.
    Note: The internal P2P preview, cloud storage, and local storage of the SDK are all use E_IPC_STREAM_AUDIO_MAIN data. */
    ipc_sdk_run_var.media_info.media_info.stream_enable[E_IPC_STREAM_AUDIO_MAIN] = TRUE;                    /* Whether to enable local sound collection */
    ipc_sdk_run_var.media_info.media_info.audio_codec[E_IPC_STREAM_AUDIO_MAIN] = TUYA_CODEC_AUDIO_PCM;      /* Encoding format */
    ipc_sdk_run_var.media_info.media_info.audio_sample[E_IPC_STREAM_AUDIO_MAIN] = TUYA_AUDIO_SAMPLE_8K;     /* Sampling Rate */
    ipc_sdk_run_var.media_info.media_info.audio_databits[E_IPC_STREAM_AUDIO_MAIN] = TUYA_AUDIO_DATABITS_16; /* Bit width */
    ipc_sdk_run_var.media_info.media_info.audio_channel[E_IPC_STREAM_AUDIO_MAIN] = TUYA_AUDIO_CHANNEL_MONO; /* channel */
    ipc_sdk_run_var.media_info.media_info.audio_fps[E_IPC_STREAM_AUDIO_MAIN] = 25;                          /* Fragments per second */

    /*local storage (custome whether enable or not)*/
    ipc_sdk_run_var.local_storage_info.enable = 1;
    ipc_sdk_run_var.local_storage_info.max_event_num_per_day = 500;
    ipc_sdk_run_var.local_storage_info.skills = TUYA_IPC_SKILL_BASIC | TUYA_IPC_SKILL_DOWNLOAD | TUYA_IPC_SKILL_SPEED_PLAY_0Point5 | TUYA_IPC_SKILL_SPEED_PLAY_2;
    
    // ipc_sdk_run_var.local_storage_info.skills = TUYA_IPC_SKILL_DOWNLOAD | TUYA_IPC_SKILL_BASIC | TUYA_IPC_SKILL_DELETE_BY_DAY | TUYA_IPC_SKILL_SPEED_PLAY_0Point5 | TUYA_IPC_SKILL_SPEED_PLAY_2 | TUYA_IPC_SKILL_SPEED_PLAY_4 | TUYA_IPC_SKILL_SPEED_PLAY_8 ;
    ipc_sdk_run_var.local_storage_info.sd_status_cb = TUYA_IPC_sd_status_upload;
    strcpy(ipc_sdk_run_var.local_storage_info.storage_path, s_ipc_sd_path);

    /*cloud_storage_info*/
    ipc_sdk_run_var.cloud_storage_info.enable = 1;
    ipc_sdk_run_var.cloud_storage_info.en_audio_record = 1;
    ipc_sdk_run_var.cloud_storage_info.pre_record_time = 2;

    /*media adapter function (essential)*/
    ipc_sdk_run_var.media_adatper_info.max_stream_client = 5;
    ipc_sdk_run_var.media_adatper_info.live_mode = TRANS_DEFAULT_STANDARD;
    ipc_sdk_run_var.media_adatper_info.media_event_cb = TUYA_IPC_p2p_event_cb;
    ipc_sdk_run_var.media_adatper_info.rev_audio_cb = TUYA_IPC_APP_rev_audio_cb;
    ipc_sdk_run_var.media_adatper_info.rev_video_cb = TUYA_IPC_APP_rev_video_cb;
    ipc_sdk_run_var.media_adatper_info.rev_file_cb = TUYA_IPC_APP_rev_file_cb;
    ipc_sdk_run_var.media_adatper_info.get_snapshot_cb = TUYA_APP_get_snapshot_cb;

    /*event function (essential)*/
    TUYA_ALARM_BITMAP_T alarm_of_events[1] = { 0 };
    tuya_ipc_event_add_alarm_types(&alarm_of_events[0], E_ALARM_MOTION);
    ipc_sdk_run_var.event_info.enable = 1;
    ipc_sdk_run_var.event_info.max_event = sizeof(alarm_of_events) / sizeof(TUYA_ALARM_BITMAP_T);
    ipc_sdk_run_var.event_info.alarms_of_events = alarm_of_events;
    ipc_sdk_run_var.event_info.alarms_cnt_per_event = 1;

    /*AI detect (custome whether enable or not)*/
    ipc_sdk_run_var.cloud_rule_info.enable = 0;
    ipc_sdk_run_var.cloud_rule_info.on_ai_result_cb = NULL;
    ipc_sdk_run_var.cloud_rule_info.on_video_msg_cb = NULL;

    /*door bell (custome whether enable or not)*/
    ipc_sdk_run_var.video_msg_info.enable = 1;
    ipc_sdk_run_var.video_msg_info.type = MSG_BOTH;
    ipc_sdk_run_var.video_msg_info.msg_duration = 10;

    /*dp function(essential)*/
    ipc_sdk_run_var.dp_info.dp_query = TUYA_IPC_handle_dp_query_objs;               // 基站设备 和 子设备 DP 通过入参区分
    ipc_sdk_run_var.dp_info.raw_dp_cmd_proc = TUYA_IPC_handle_raw_dp_cmd_objs;      // 基站设备 和 子设备 DP 通过入参区分
    ipc_sdk_run_var.dp_info.common_dp_cmd_proc = TUYA_IPC_handle_dp_cmd_objs;       // 基站设备 和 子设备 DP 通过入参区分

    /*upgrade function(essential)*/
    ipc_sdk_run_var.upgrade_info.enable = true;
    ipc_sdk_run_var.upgrade_info.upgrade_cb = TUYA_IPC_Upgrade_Inform_cb;
    strcpy(ipc_sdk_run_var.upgrade_info.upgrade_file, s_ipc_upgrade_file);

    ipc_sdk_run_var.iot_info.gw_reset_cb = TUYA_IPC_Reset_System_CB;
    ipc_sdk_run_var.iot_info.gw_restart_cb = TUYA_IPC_Restart_Process_CB;

    /*QR-active function(essential)*/
    ipc_sdk_run_var.qrcode_active_cb = NULL; // TUYA_IPC_qrcode_shorturl_cb;

    /* rtc call function(essential), only use in two-way video talk */
    ipc_sdk_run_var.call_info.enable = FALSE;

    OPERATE_RET ret;
    ret = tuya_ipc_app_start(&ipc_sdk_run_var);
    if (ret != 0) {
        PR_DEBUG("ipc sdk start fail,please check run parameter,ret=%d\n", ret);
    }
    if (TUYA_IPC_LINK_WIRE == link_type) {
        // if only use wire for paring
        tuya_svc_netmgr_linkage_set_default(LINKAGE_TYPE_WIRED);
    }
    return ret;
}

static OPERATE_RET TUYA_APP_Init_Stream_Storage(TUYA_IPC_SDK_LOCAL_STORAGE_S* p_local_storage_info)
{
    STATIC BOOL_T s_stream_storage_inited = FALSE;
    INT_T i = 0;

    if (s_stream_storage_inited == TRUE) {
        PR_DEBUG("The Stream Storage Is Already Inited");
        return OPRT_OK;
    }

    if (p_local_storage_info == NULL) {
        PR_DEBUG("Init Stream Storage fail. Param is null");
        return OPRT_INVALID_PARM;
    }

    // 由开发者管理 "全局存储模式" 配置项
    // SDK存储内部通过 tuya_ipc_sd_get_mode_config 自动获取模式
    IPC_APP_set_storage_mode_config(SS_WRITE_MODE_NONE);

    TUYA_IPC_STORAGE_VAR_T stg_var;
    memset(&stg_var, 0, SIZEOF(TUYA_IPC_STORAGE_VAR_T));

    stg_var.camera_device_max_count = 4;    // 设置最大 (带有音视频功能) 子设备数量
    memcpy(stg_var.base_path, p_local_storage_info->storage_path, strlen(p_local_storage_info->storage_path));      // 存储路径
    stg_var.max_event_per_day = p_local_storage_info->max_event_num_per_day;
    stg_var.sd_status_changed_cb = p_local_storage_info->sd_status_cb;
    stg_var.skills = p_local_storage_info->skills;

    UINT_T arr_sz = stg_var.camera_device_max_count * sizeof(STREAM_STORAGE_MEDIA_INFO_T);
    STREAM_STORAGE_MEDIA_INFO_T* p_device_media_info = malloc(arr_sz);
    if (!p_device_media_info) {
        PR_ERR("malloc err");
        return OPRT_COM_ERROR;
    }
    memset(p_device_media_info, 0, arr_sz);

    // 已添加的设备，需要在初始化时传入音视频参数 
    // 在程序运行中添加的子设备，需要在添加成功后调用 tuya_ipc_ss_add_new_node_storage_by_sub_node_id 开启子设备的存储，存储方式默认为全局设置值
    for (i = 0; i < stg_var.camera_device_max_count; i++) {
        IPC_MEDIA_INFO_T param;
        memset(&param, 0, sizeof(param));
        TUYA_APP_Get_Media_Info(i, &param);

        p_device_media_info[i].video_enable = 1;
        p_device_media_info[i].video_fps = param.video_fps[E_IPC_STREAM_VIDEO_MAIN];
        p_device_media_info[i].video_gop = param.video_gop[E_IPC_STREAM_VIDEO_MAIN];
        p_device_media_info[i].video_bitrate = param.video_bitrate[E_IPC_STREAM_VIDEO_MAIN];
        p_device_media_info[i].video_width = param.video_width[E_IPC_STREAM_VIDEO_MAIN];
        p_device_media_info[i].video_height = param.video_height[E_IPC_STREAM_VIDEO_MAIN];
        p_device_media_info[i].video_freq = param.video_freq[E_IPC_STREAM_VIDEO_MAIN];
        p_device_media_info[i].video_codec = param.video_codec[E_IPC_STREAM_VIDEO_MAIN];

        p_device_media_info[i].audio_enable = 1;
        p_device_media_info[i].audio_codec = param.audio_codec[E_IPC_STREAM_AUDIO_MAIN];
        p_device_media_info[i].audio_fps = param.audio_fps[E_IPC_STREAM_AUDIO_MAIN];
        p_device_media_info[i].audio_sample = param.audio_sample[E_IPC_STREAM_AUDIO_MAIN];
        p_device_media_info[i].audio_databits = param.audio_databits[E_IPC_STREAM_AUDIO_MAIN];
        p_device_media_info[i].audio_channel = param.audio_channel[E_IPC_STREAM_AUDIO_MAIN];
    }
    stg_var.device_media_info_arr = p_device_media_info;

    PR_DEBUG("Init Stream_Storage SD:%s", p_local_storage_info->storage_path);
    OPERATE_RET ret = tuya_ipc_ss_init(&stg_var);
    free(p_device_media_info);

    if (ret != OPRT_OK) {
        PR_ERR("Init Main Video Stream_Storage Fail. %d", ret);
        return OPRT_COM_ERROR;
    }

    return OPRT_OK;
}

#if defined(ENABLE_CLOUD_RULE) && (ENABLE_CLOUD_RULE == 1)
static OPERATE_RET TUYA_APP_Init_Cloud_Rule(TUYA_IPC_SDK_CLOUD_RULE_S* p_cloud_rule)
{
    tuya_ipc_rule_init((RULE_AI_RESULT_CB)p_cloud_rule->on_ai_result_cb, (RULE_VIDEO_MSG_CB)p_cloud_rule->on_video_msg_cb);
}
#endif

static OPERATE_RET TUYA_APP_Enable_CloudStorage(TUYA_IPC_SDK_CLOUD_STORAGE_S* p_cloud_storage_info)
{
    OPERATE_RET ret;
    INT_T i = 0;
    INT_T bind_cnt = 0;
    INT_T dev_arr[SUB_DEV_MAX_NUM] = {0};

    ret = tuya_xvr_cloud_storage_init();
    if (ret != OPRT_OK) {
        PR_DEBUG("Cloud Storage Init Err! ret :%d", ret);
        return ret;
    }

    if (0 == tuya_xvr_dev_get_all_sub_nodes_rinfbuf_idx(&bind_cnt, dev_arr)) {
        for (i = 0; i < bind_cnt; i++) {
            TUYA_XVR_DEV_INFO_T sub_dev_info = {0};
            memset(&sub_dev_info, 0, sizeof(sub_dev_info));
            tuya_xvr_dev_get_sub_dev_info_by_ringbuf_idx(dev_arr[i], &sub_dev_info);

            ret = tuya_xvr_cloud_storage_task_start(sub_dev_info.sub_node_id);
            PR_DEBUG("start sub dev cloud storage task ret=%d, sub_id=%s", ret, sub_dev_info.sub_node_id);
        }
    }

    return OPRT_OK;
}

static void __ai_send_audio_file_data(char* filename)
{
    const size_t sample_rate = 16000;                                        // 16KHz采样率
    const size_t bit_depth = 16;                                             // 16位采样
    const size_t frame_duration_ms = 20;                                     // 每帧20毫秒
    const size_t samples_per_frame = sample_rate * frame_duration_ms / 1000; // 每帧的样本数
    const size_t frame_size_bytes = samples_per_frame * (bit_depth / 8);     // 每帧的字节数
    TKL_AUDIO_FRAME_INFO_T audioFrame = { 0 };

    audioFrame.type = TKL_AUDIO_FRAME;
    audioFrame.codectype = TKL_CODEC_AUDIO_PCM;
    audioFrame.sample = TKL_AUDIO_SAMPLE_16K;
    audioFrame.datebits = TKL_AUDIO_DATABITS_16;
    audioFrame.channel = TKL_AUDIO_CHANNEL_MONO;

    // 打开PCM文件
    FILE* file = fopen(filename, "rb");
    if (!file) {
        perror("Failed to open PCM file");
        return;
    }

    // 分配内存用于存储一帧PCM数据
    char* pcm_frame = (char*)malloc(frame_size_bytes);
    if (!pcm_frame) {
        perror("Failed to allocate memory for PCM frame");
        fclose(file);
        return;
    }

    // 读取文件并发送每一帧
    while (1) {
        // 从文件中读取一帧数据
        size_t bytes_read = fread(pcm_frame, 1, frame_size_bytes, file);
        // 如果读取的字节数不足一帧，说明文件已经读完
        if (bytes_read < frame_size_bytes) {
            if (bytes_read > 0) {
                // 发送剩余的数据（不足一帧）
                audioFrame.pbuf = pcm_frame;
                audioFrame.buf_size = frame_size_bytes;
                audioFrame.used_size = bytes_read;
                ty_ai_toy_audio_frame_put(&audioFrame);
            }
            break; // 退出循环
        }
        // 发送完整的一帧
        audioFrame.pbuf = pcm_frame;
        audioFrame.buf_size = frame_size_bytes;
        audioFrame.used_size = frame_size_bytes;
        ty_ai_toy_audio_frame_put(&audioFrame);

        usleep(frame_duration_ms * 1000);
    }

    // 释放内存并关闭文件
    free(pcm_frame);
    fclose(file);

    return;
}

STATIC OPERATE_RET __bind_result_cb(void *data)
{
    TUYA_XVR_DEV_BIND_RESULT_T* bind_result = (TUYA_XVR_DEV_BIND_RESULT_T*)data;
    PR_DEBUG("===> sub dev[%s] result is %d\n", bind_result->sub_node_id, bind_result->result);
    // 1.判断添加结果


    // 2. 拿到子设备对应的序号, 在添加成功且基站设备不断电时, 全局对应关系是唯一的
    // 开发者需要注意 device_no 和 sub_node_id 关系持续对应
    // 子设备 sub_node_id 唯一不变
    // !!! 使用 tuya_xvr_dev_get_ringbuf_idx_by_sub_node_id 获取子设备编号进行上层业务
    INT_T device_no = 0;
    INT_T ret = tuya_xvr_dev_get_ringbuf_idx_by_sub_node_id(bind_result->sub_node_id, &device_no);
    if (ret != OPRT_OK) {
        PR_ERR("get subdev:%s device number err:%d", bind_result->sub_node_id, device_no);
        return OPRT_COM_ERROR;
    }

    // 由开发者管理子设备device_no和对应的媒体信息
    IPC_MEDIA_INFO_T media_info = { 0 };
    TUYA_APP_Get_Media_Info(device_no, &media_info);

    DEVICE_MEDIA_INFO_T device_media_info = { 0 };
    memcpy(&device_media_info.av_encode_info, &media_info, sizeof(IPC_MEDIA_INFO_T));
    device_media_info.audio_decode_info.enable = 1;
    device_media_info.audio_decode_info.audio_codec = TUYA_CODEC_AUDIO_G711U;
    device_media_info.audio_decode_info.audio_sample = TUYA_AUDIO_SAMPLE_8K;
    device_media_info.audio_decode_info.audio_databits = TUYA_AUDIO_DATABITS_16;
    device_media_info.audio_decode_info.audio_channel = TUYA_AUDIO_CHANNEL_MONO;
    device_media_info.max_pic_len = 100 * 1024;

    // 3. 设置子设备媒体信息 到 SDK
    tuya_ipc_media_adapter_set_media_info(device_no, 0, device_media_info);

    // 4. 初始化 子设备ringbuf
    PR_DEBUG("sub dev id[%s] ringbuf_idx[%d] ", bind_result->sub_node_id, device_no);
    TUYA_APP_Init_Ring_Buffer(&media_info, device_no);

    // 5. 由开发者管理 子设备码流输入 和 写入ringbuf 的时机
    TUYA_APP_set_dev_enable(device_no);

    // 6. 开启子设备本地存储
    STREAM_STORAGE_MEDIA_INFO_T ss_media_info;
    memset(&ss_media_info, 0, sizeof(ss_media_info));

    ss_media_info.video_enable = 1;
    ss_media_info.video_fps = media_info.video_fps[E_IPC_STREAM_VIDEO_MAIN];
    ss_media_info.video_gop = media_info.video_gop[E_IPC_STREAM_VIDEO_MAIN];
    ss_media_info.video_bitrate = media_info.video_bitrate[E_IPC_STREAM_VIDEO_MAIN];
    ss_media_info.video_width = media_info.video_width[E_IPC_STREAM_VIDEO_MAIN];
    ss_media_info.video_height = media_info.video_height[E_IPC_STREAM_VIDEO_MAIN];
    ss_media_info.video_freq = media_info.video_freq[E_IPC_STREAM_VIDEO_MAIN];
    ss_media_info.video_codec = media_info.video_codec[E_IPC_STREAM_VIDEO_MAIN];

    ss_media_info.audio_enable = 1;
    ss_media_info.audio_codec = media_info.audio_codec[E_IPC_STREAM_AUDIO_MAIN];
    ss_media_info.audio_fps = media_info.audio_fps[E_IPC_STREAM_AUDIO_MAIN];
    ss_media_info.audio_sample = media_info.audio_sample[E_IPC_STREAM_AUDIO_MAIN];
    ss_media_info.audio_databits = media_info.audio_databits[E_IPC_STREAM_AUDIO_MAIN];
    ss_media_info.audio_channel = media_info.audio_channel[E_IPC_STREAM_AUDIO_MAIN];

    tuya_ipc_ss_add_new_node_storage_by_sub_node_id(bind_result->sub_node_id, &ss_media_info);

    return 0;
}

int TUYA_APP_Add_Xvr_Sub_Dev()
{
    INT_T i = 0;
    // step:当前主要以NVR模式绑定子设备。确保绑定后，不再重复绑定。
    //  若nvr还没添加子设备，则添加下
    TUYA_XVR_BIND_CTX_T bind_ctx = { 0 };

    // 在 __bind_result_cb 中收到子设备添加成功的消息后再进行后续操作
    OPERATE_RET ret = tuya_xvr_dev_attach_sub_dev_bind_result_cb(__bind_result_cb);
    PR_DEBUG("===> attach ret result is %d\n", ret);

    for (i = 0; i < MAX_DEV_NUM; i++) {
        if (0 == sg_sub_enable[i]) {
            continue;
        }
        
        bind_ctx.tp = DEV_NM_ATH_SNGL;      // DEV_NM_ATH_SNGL 表示(带有音视频输出功能的)子设备类型, SDK内部会判断此类型
        bind_ctx.uddd = USER_DEV_DTL_IPC;   // udd is ipc , 用户可以自定义，用于自己标记是什么设备类型
        strncpy(bind_ctx.sub_node_id, sg_sub_ipc_uuid[i], sizeof(bind_ctx.sub_node_id) - 1);
        strncpy(bind_ctx.product_id, sg_nvr_sub_pid[i], sizeof(bind_ctx.product_id) - 1);
        strncpy(bind_ctx.version, s_app_version, sizeof(bind_ctx.product_id) - 1);
        tuya_xvr_dev_bind(&bind_ctx);
    }

    return 0;
}

int TUYA_APP_Get_Xvr_Sub_Dev()
{
    INT_T i = 0;
    for (i = 0; i < MAX_DEV_NUM; i++) {
        if (0 == sg_sub_enable[i]) {
            continue;
        }

        DEV_DESC_IF_S* pDev = tuya_xvr_dev_get_sub_dev_desc_info(sg_sub_ipc_uuid[i]);
        if (NULL != pDev) {
            PR_DEBUG("id[%s] virt_id[%s]\n", pDev->id, pDev->virt_id);
        } else {
            PR_DEBUG("pDev\n");
        }
        char virId[32] = "";
        tuya_xvr_dev_get_dev_id_by_sub_node_id(sg_sub_ipc_uuid[i], virId, sizeof(virId));
        int idx = -1;
        tuya_xvr_dev_get_ringbuf_idx_by_sub_node_id(sg_sub_ipc_uuid[i], &idx);
        PR_DEBUG("id[%s] virt_id[%s] idx[%d]\n", pDev->id, virId, idx);
    }

    return 0;
}

int TUYA_APP_Remove_Xvr_Sub_Dev()
{
    INT_T i = 0;
    for (i = 0; i < MAX_DEV_NUM; i++) {
        if (0 == sg_sub_enable[i]) {
            continue;
        }
        tuya_xvr_dev_remove_sub_node(sg_sub_ipc_uuid[i]);
    }

    return 0;
}

int main(int argc, char** argv)
{
    INT_T res = -1;
    TUYA_IPC_PARING_MODE_E mode = PARING_MODE_DEBUG;
    char cfg_path[256] = { 0 };

    //测试子设备添加使用，用户可以根据自己的需求修改子设备的PID UUID AUTHKEY，调用部分参考 TUYA_APP_Add_Xvr_Sub_Dev
    sg_sub_enable[0] = 0;
    strncpy(sg_nvr_sub_pid[0], "iucw1vxrjaajk9bh", sizeof(sg_nvr_sub_pid[0]) - 1);
    strncpy(sg_sub_ipc_uuid[0], "", sizeof(sg_sub_ipc_uuid[0]) - 1);
    strncpy(sg_sub_ipc_authkey[0], "", sizeof(sg_sub_ipc_authkey[0]) - 1);

    sg_sub_enable[1] = 0;
    strncpy(sg_nvr_sub_pid[1], "iucw1vxrjaajk9bh", sizeof(sg_nvr_sub_pid[1]) - 1);
    strncpy(sg_sub_ipc_uuid[1], "", sizeof(sg_sub_ipc_uuid[1]) - 1);
    strncpy(sg_sub_ipc_authkey[1], "", sizeof(sg_sub_ipc_authkey[1]) - 1);

    sg_sub_enable[2] = 0;
    strncpy(sg_nvr_sub_pid[2], "iucw1vxrjaajk9bh", sizeof(sg_nvr_sub_pid[2]) - 1);
    strncpy(sg_sub_ipc_uuid[2], "", sizeof(sg_sub_ipc_uuid[2]) - 1);
    strncpy(sg_sub_ipc_authkey[2], "", sizeof(sg_sub_ipc_authkey[2]) - 1);

    sg_sub_enable[3] = 0;
    strncpy(sg_nvr_sub_pid[3], "iucw1vxrjaajk9bh", sizeof(sg_nvr_sub_pid[3]) - 1);
    strncpy(sg_sub_ipc_uuid[3], "", sizeof(sg_sub_ipc_uuid[3]) - 1);
    strncpy(sg_sub_ipc_authkey[3], "", sizeof(sg_sub_ipc_authkey[3]) - 1);

    while ((res = getopt(argc, argv, "?m:t:s:r:c:h")) != -1) {
        switch (res) {
        case 'm':
            mode = atoi(optarg);
            break;
        case 't':
            strcpy(s_token, optarg);
            break;
        case 'r':
            strcpy(s_raw_path, optarg);
            break;
        case 'c':
            snprintf(cfg_path, 256, "%s/tuya_device_cfg.json", optarg);
            break;
        case 'h':
        default:
            __USAGE(argv[0]);
            return -1;
        }
    }

    printf("[%s, %d] %s,%s token[%s] raw_path[%s] cfg_path[%s]\n", __FUNCTION__, __LINE__, __DATE__, __TIME__, s_token, s_raw_path, cfg_path);

    if (0 != __TUYA_IPC_DEMO_CFG_PARSE(cfg_path)) {
        printf("media cfg parse failed\n");
        return -1;
    }
    TUYA_IPC_INIT_DP_CONFIG();

    signal(SIGINT, signal_handle);
    signal(SIGKILL, signal_handle);
    signal(SIGTERM, signal_handle);
    signal(SIGPIPE, SIG_IGN);

    // If set to 1, the device will work in lan only mode
    tuya_set_lan_mode(0);

    res = __TUYA_IPC_SDK_START(mode, s_token);
    if (res != OPRT_OK) {
        return res;
    }

    TUYA_APP_Enable_Motion_Detect();

    if (tuya_get_lan_mode() != 1) {
       /* whether SDK is connected to MQTT */
        while (TUYA_IPC_Get_MqttStatus() == FALSE) {
            tal_system_sleep(1000);
        }
        PR_DEBUG("tuya_ipc_sdk_mqtt_online_proc is start run\n");
        //同步服务器时间
        TIME_T time_utc;
        INT_T time_zone;
        do {
            //需要SDK同步到时间后才能开启下面的业务
            res = tuya_ipc_get_service_time_force(&time_utc, &time_zone);
        } while (res != OPRT_OK);
    } else {
        //免配网模式使用，大多数场景不用关注
        if (tuya_ipc_get_register_status() != GW_BLE_ACTIVED) {
            // Device not activated, waiting for activation
            while (tuya_ipc_get_register_status() != GW_BLE_ACTIVED) {
                tal_system_sleep(1000);
            }
            // After activation, the APP will synchronize time with the device
            TIME_T time_utc;
            INT_T time_zone;
            while (tuya_ipc_get_service_time(&time_utc, &time_zone) != 0) {
                tal_system_sleep(100);
            }
        } else {
            // Activated devices no longer synchronize time. The time needs to be retrieved from RTC or other methods
        }
 
    }

    TUYA_IPC_av_start();

    if (FALSE == g_sdk_run_info.quick_start_info.enable) {
        TUYA_IPC_Media_Stream_Init(&(g_sdk_run_info.media_adatper_info));
    }

    if (g_sdk_run_info.event_info.enable) {
        res = tuya_ipc_event_module_init(g_sdk_run_info.event_info.max_event,
                                         g_sdk_run_info.event_info.alarms_of_events, g_sdk_run_info.event_info.alarms_cnt_per_event);
        PR_DEBUG("event module init result is %d\n", res);
    }

    if (g_sdk_run_info.local_storage_info.enable) {
        res = TUYA_APP_Init_Stream_Storage(&(g_sdk_run_info.local_storage_info));
        PR_DEBUG("local storage init result is %d\n", res);
    }
#if defined(ENABLE_CLOUD_RULE) && (ENABLE_CLOUD_RULE == 1)
    if (g_sdk_run_info.cloud_rule_info.enable) {
        res = TUYA_APP_Init_Cloud_Rule(&(g_sdk_run_info.cloud_rule_info));
        PR_DEBUG("cloud rule init result is %d\n", res);
    }
#endif
    if (g_sdk_run_info.cloud_storage_info.enable) {
        res = TUYA_APP_Enable_CloudStorage(&(g_sdk_run_info.cloud_storage_info));
        PR_DEBUG("cloud storage init result is %d\n", res);
    }
#if defined(ENABLE_TUYA_CALL) && (ENABLE_TUYA_CALL == 1)
    if (g_sdk_run_info.call_info.enable) {
        TUYA_IPC_call_init();
    }
#endif
    TUYA_IPC_upload_all_status();

    tuya_ipc_upload_skills();
    PR_DEBUG("tuya_ipc_sdk_mqtt_online_proc is end run\n");

    init_rtsp_client();

    TUYA_IPC_simulation();

    printf("[%s, %d]\n", __FUNCTION__, __LINE__);

    return 0;
}
