#include <libgen.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/prctl.h>
#include <unistd.h>

#include "rtsp_client.h"
#include "tal_mutex.h"
#include "tuya_iot_config.h"
#include "utilities/uni_log.h"

#define RTSP_CLIENT_URL "rtsp://192.168.16.92:8554/stream0"

typedef struct {
    FILE* write_fd;
    uint32_t write_sz;
    int32_t close_flag;

    uint32_t frame_rev_sz;
    char frame_buf[512 * 1024];
} RTSP_CLIENT_TEST_INFO_T;

static MUTEX_HANDLE sg_mutex;
static RTSP_CLIENT_HANDLE_T sg_rtsp_client_hdl;
static RTSP_CLIENT_TEST_INFO_T* sg_rtsp_client_info;

INT_T rtsp_client_put_data_cb(RTSP_CLIENT_HANDLE_T handle, RTSP_DATA_PACKET_T* pkt, VOID* priv)
{
    int32_t ret = 0;
    RTSP_CLIENT_TEST_INFO_T* hdl = (RTSP_CLIENT_TEST_INFO_T*)priv;

    if (E_RTSP_AUDIO_FRAME == pkt->type) {
        // 音频
        return 0;
    }

    tal_mutex_lock(sg_mutex);

    if (1 == hdl->close_flag) {
        tal_mutex_unlock(sg_mutex);
        return -1;
    }

    if (0 != pkt->frame_start) {
        uint8_t flag[4] = { 0 };
        flag[3] = 1;
        memcpy(hdl->frame_buf + hdl->frame_rev_sz, flag, sizeof(flag));
        hdl->frame_rev_sz += 4;
    }

    memcpy(hdl->frame_buf + hdl->frame_rev_sz, pkt->buf, pkt->size);
    hdl->frame_rev_sz += pkt->size;
    PR_ERR("recv data:%llu,%u,[%d,%d]", pkt->timestamp, pkt->size,
           pkt->frame_start, pkt->frame_end);

    // VPS SPS PPS IDR 要合在一起作为一个 I 帧写入ringbuf
    if (0 != pkt->frame_end
        && (E_RTSP_VIDEO_I_FRAME == pkt->type || E_RTSP_VIDEO_PB_FRAME == pkt->type)) {
        if (!hdl->write_fd) {
            hdl->write_fd = fopen("./test_media", "w+");
        }

        if (hdl->write_fd) {
            ret = fwrite(hdl->frame_buf, 1, hdl->frame_rev_sz, hdl->write_fd);
            fflush(hdl->write_fd);

            hdl->write_sz += hdl->frame_rev_sz;
            PR_ERR("write frame:%llu,typ:%d,sz:%d,write:%d,write-tot:%u", pkt->timestamp, pkt->type, hdl->frame_rev_sz,
                   ret, hdl->write_sz);
        }
        hdl->frame_rev_sz = 0;
    }

    tal_mutex_unlock(sg_mutex);

    return ret;
}

INT_T rtsp_client_status_change_cb(RTSP_CLIENT_HANDLE_T handle, RTSP_CLIENT_STATUS_E status, VOID* priv)
{
    PR_ERR("rtsp_client_status:%d", status);

    return 0;
}

void init_rtsp_client()
{
    sg_rtsp_client_info = malloc(sizeof(RTSP_CLIENT_TEST_INFO_T));
    if (!sg_rtsp_client_info) {
        PR_ERR("malloc err");

        return;
    }

    tal_mutex_create_init(&sg_mutex);

    RTSP_CLIENT_INIT_PARAM_T param = {
        .stack_size = 100 * 1024,
    };

    int ret = tuya_ipc_rtsp_client_init(&param);
    PR_ERR("tuya_ipc_rtsp_client_init ret:%d", ret);
}

void start_rtsp_client()
{
    memset(sg_rtsp_client_info, 0, sizeof(RTSP_CLIENT_TEST_INFO_T));

    RTSP_CLIENT_PARAM_T st_param;
    memset(&st_param, 0, sizeof(st_param));

    memcpy(st_param.url, RTSP_CLIENT_URL, strlen(RTSP_CLIENT_URL));
    memcpy(st_param.username, "admin", strlen("admin"));
    memcpy(st_param.password, "Admin123", strlen("Admin123"));
    st_param.trans_mode = RTSP_TRANSPORT_MODE_TCP;
    st_param.put_data_cb = rtsp_client_put_data_cb;
    st_param.status_change_cb = rtsp_client_status_change_cb;
    st_param.priv = sg_rtsp_client_info;

    sg_rtsp_client_hdl = tuya_ipc_rtsp_client_start(&st_param);
}

void stop_rtsp_client()
{
    tuya_ipc_rtsp_client_stop(sg_rtsp_client_hdl);

    tal_mutex_lock(sg_mutex);

    sg_rtsp_client_info->close_flag = 1;
    fflush(sg_rtsp_client_info->write_fd);
    fclose(sg_rtsp_client_info->write_fd);
    sg_rtsp_client_info->write_fd = NULL;

    tal_mutex_unlock(sg_mutex);
}
