/*********************************************************************************
 *Copyright(C),2015-2020,
 *TUYA
 *FileName: tuya_ipc_dp_utils.c
 *
 * File Description：
 * 1. API implementation of DP point
 *
 * This file code is the basic code, users don't care it
 * Please do not modify any contents of this file at will.
 * Please contact the Product Manager if you need to modify it.
 *
 **********************************************************************************/
#include "tuya_cloud_com_defs.h"
#include "tuya_iot_config.h"
#include "tuya_ipc_api.h"
#include "ty_dp_define.h"
#include "ty_sdk_common.h"
#include "utilities/uni_log.h"
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

STATIC VOID respone_dp_value(BYTE_T dp_id, INT_T val);
STATIC VOID respone_dp_bool(BYTE_T dp_id, BOOL_T true_false);
STATIC VOID respone_dp_enum(BYTE_T dp_id, CHAR_T* p_val_enum);
STATIC VOID respone_dp_str(BYTE_T dp_id, CHAR_T* p_val_str);


//------------------------------------------
STATIC VOID respone_dp_value(BYTE_T dp_id, INT_T val)
{
    tuya_ipc_dp_report(NULL, dp_id, PROP_VALUE, &val, 1);
}

STATIC VOID respone_dp_bool(BYTE_T dp_id, BOOL_T true_false)
{
    tuya_ipc_dp_report(NULL, dp_id, PROP_BOOL, &true_false, 1);
}

STATIC VOID respone_dp_enum(BYTE_T dp_id, CHAR_T* p_val_enum)
{
    tuya_ipc_dp_report(NULL, dp_id, PROP_ENUM, p_val_enum, 1);
}

STATIC VOID respone_dp_str(BYTE_T dp_id, CHAR_T* p_val_str)
{
    tuya_ipc_dp_report(NULL, dp_id, PROP_STR, p_val_str, 1);
}

//------------------------------------------
STATIC BOOL_T check_dp_bool_invalid(IN TY_OBJ_DP_S* p_obj_dp)
{
    if (p_obj_dp == NULL) {
        PR_ERR("error! input is null \r\n");
        return -1;
    }

    if (p_obj_dp->type != PROP_BOOL) {
        PR_ERR("error! input is not bool %d \r\n", p_obj_dp->type);
        return -2;
    }

    if (p_obj_dp->value.dp_bool == 0) {
        return FALSE;
    } else if (p_obj_dp->value.dp_bool == 1) {
        return TRUE;
    } else {
        PR_ERR("Error!! type invalid %d \r\n", p_obj_dp->value.dp_bool);
        return -2;
    }
}

VOID IPC_APP_report_sd_storage()
{
    CHAR_T tmp_str[100] = { 0 };

    UINT_T total = 100;
    UINT_T used = 0;
    UINT_T empty = 100;
    IPC_APP_get_sd_storage(&total, &used, &empty);

    //"total capacity|Current usage|remaining capacity"
    snprintf(tmp_str, 100, "%u|%u|%u", total, used, empty);
    respone_dp_str(TUYA_DP_STOR_VOLUME, tmp_str);
}
//------------------------------------------

VOID IPC_APP_report_sd_format_status(INT_T status)
{
    respone_dp_value(TUYA_DP_STOR_FORMAT_STATUS, status);
}

VOID IPC_APP_report_sd_status_changed(INT_T status)
{
    respone_dp_value(TUYA_DP_STOR_STATUS, status);
}

VOID IPC_APP_update_battery_status(VOID)
{
    CHAR_T power_mode[4] = { 0 };
    snprintf(power_mode, 4, "%d", IPC_APP_get_power_mode());
    INT_T percent = IPC_APP_get_battery_percent();

    PR_INFO("current power mode:%s\r\n", power_mode);
    respone_dp_enum(TUYA_DP_POWERMODE, power_mode);
    PR_INFO("current battery percent:%d\r\n", percent);
    respone_dp_value(TUYA_DP_ELECTRICITY, percent);
}




////////////////////////////////////////////////////////////////////////////////////


STATIC VOID handle_DP_ELECTRICITY(IN TY_OBJ_DP_S* p_obj_dp)
{
    INT_T percent = IPC_APP_get_battery_percent();
    PR_INFO("current battery percent:%d\r\n", percent);
    respone_dp_value(TUYA_DP_ELECTRICITY, percent);
}

STATIC VOID handle_DP_POWERMODE(IN TY_OBJ_DP_S* p_obj_dp)
{
    CHAR_T* power_mode = IPC_APP_get_power_mode();
    PR_INFO("current power mode:%s\r\n", power_mode);
    respone_dp_enum(TUYA_DP_POWERMODE, power_mode);
}

VOID handle_DP_SD_STORAGE_VOLUME_GET(IN TY_OBJ_DP_S* p_obj_dp)
{
    IPC_APP_report_sd_storage();
}

STATIC VOID handle_DP_SD_STATUS_GET(IN TY_OBJ_DP_S* p_obj_dp)
{
    /* SD card status, VALUE type, 1-normal, 2-anomaly, 3-insufficient space, 4-formatting, 5-no SD card */
    respone_dp_value(TUYA_DP_STOR_STATUS, 1);
}

STATIC VOID handle_DP_SD_FORMAT(IN TY_OBJ_DP_S* p_obj_dp)
{
    extern VOID IPC_APP_format_sd_card(VOID);
    IPC_APP_format_sd_card();
}

STATIC VOID handle_DP_SD_FORMAT_STATUS_ONLY_GET(IN TY_OBJ_DP_S* p_obj_dp)
{
    respone_dp_value(TUYA_DP_STOR_FORMAT_STATUS, 1);
}

static int s_gcur_storage_mode = 0x1;
STATIC VOID handle_DP_SET_CUR_STORAGE(IN TY_OBJ_DP_S* p_obj_dp)
{
    s_gcur_storage_mode = p_obj_dp->value.dp_value;
    respone_dp_value(TUYA_DP_SET_CUR_STORAGE, s_gcur_storage_mode);
}

STATIC VOID handle_DP_SUPPORT_STORAGE(IN TY_OBJ_DP_S* p_obj_dp)
{
    respone_dp_value(TUYA_DP_SUPPORT_STORAGE, s_gcur_storage_mode);
}

typedef VOID (*TUYA_DP_HANDLER)(IN TY_OBJ_DP_S* p_obj_dp);
typedef struct
{
    BYTE_T dp_id;
    TUYA_DP_HANDLER handler;
} TUYA_DP_INFO_S;

STATIC TUYA_DP_INFO_S s_dp_table[] = {

#ifdef TUYA_DP_SUPPORT_STORAGE
    { TUYA_DP_SUPPORT_STORAGE, handle_DP_SUPPORT_STORAGE },
#endif

#ifdef TUYA_DP_SET_CUR_STORAGE
    { TUYA_DP_SET_CUR_STORAGE, handle_DP_SET_CUR_STORAGE },
#endif

#ifdef TUYA_DP_STOR_VOLUME
    { TUYA_DP_STOR_VOLUME, handle_DP_SD_STORAGE_VOLUME_GET },
#endif

#ifdef TUYA_DP_STOR_STATUS
    { TUYA_DP_STOR_STATUS, handle_DP_SD_STATUS_GET },
#endif

#ifdef TUYA_DP_STOR_FORMAT
    { TUYA_DP_STOR_FORMAT, handle_DP_SD_FORMAT },
#endif

#ifdef TUYA_DP_STOR_FORMAT_STATUS
    { TUYA_DP_STOR_FORMAT_STATUS, handle_DP_SD_FORMAT_STATUS_ONLY_GET },
#endif

};

typedef VOID (*TUYA_RAW_DP_HANDLER)(IN TY_RECV_RAW_DP_S* p_obj_dp);
typedef struct
{
    BYTE_T dp_id;
    TUYA_RAW_DP_HANDLER handler;
} TUYA_RAW_DP_INFO_S;

STATIC TUYA_RAW_DP_INFO_S s_raw_dp_table[] = {
#ifdef TUYA_RAW_DP_DEMO_STYLE
    { TUYA_RAW_DP_DEMO_STYLE, handle_RAW_DP_DEMO_STYLE },
#endif

};

VOID TUYA_IPC_handle_dp_cmd_objs(IN CONST TY_RECV_OBJ_DP_S* dp_rev)
{
    // 基站设备 和 子设备 DP 通过入参区分
    // if(NULL == dp_query->cid) then the cid represents gwid
    
    TY_OBJ_DP_S* dp_data = (TY_OBJ_DP_S*)(dp_rev->dps);
    UINT_T cnt = dp_rev->dps_cnt;
    INT_T table_idx = 0;
    INT_T table_count = (sizeof(s_dp_table) / sizeof(s_dp_table[0]));
    INT_T index = 0;
    for (index = 0; index < cnt; index++) {
        TY_OBJ_DP_S* p_dp_obj = dp_data + index;

        for (table_idx = 0; table_idx < table_count; table_idx++) {
            if (s_dp_table[table_idx].dp_id == p_dp_obj->dpid) {
                s_dp_table[table_idx].handler(p_dp_obj);
                break;
            }
        }
    }
}
VOID TUYA_IPC_handle_raw_dp_cmd_objs(IN CONST TY_RECV_RAW_DP_S* dp_rev)
{
    // 基站设备 和 子设备 DP 通过入参区分
    // if(NULL == dp_query->cid) then the cid represents gwid

    INT_T table_idx = 0;
    INT_T table_count = (sizeof(s_raw_dp_table) / sizeof(s_raw_dp_table[0]));
    INT_T index = 0;

    for (table_idx = 0; table_idx < table_count; table_idx++) {
        if (s_raw_dp_table[table_idx].dp_id == dp_rev->dpid) {
            s_raw_dp_table[table_idx].handler(dp_rev);
            break;
        }
    }
}

VOID TUYA_IPC_handle_dp_query_objs(IN CONST TY_DP_QUERY_S* dp_query)
{
    // 基站设备 和 子设备 DP 通过入参区分
    // if(NULL == dp_query->cid) then the cid represents gwid

    INT_T table_idx = 0;
    INT_T table_count = (sizeof(s_dp_table) / sizeof(s_dp_table[0]));
    INT_T index = 0;
    for (index = 0; index < dp_query->cnt; index++) {
        for (table_idx = 0; table_idx < table_count; table_idx++) {
            if (s_dp_table[table_idx].dp_id == dp_query->dpid[index]) {
                s_dp_table[table_idx].handler(NULL);
                break;
            }
        }
    }
}




//------------------------------------------
VOID TUYA_IPC_upload_all_status(VOID)
{
    respone_dp_value(TUYA_DP_STOR_STATUS, 1);

    respone_dp_str(TUYA_DP_STOR_VOLUME, "128000000|32000000|96000000");

    respone_dp_value(TUYA_DP_SUPPORT_STORAGE, s_gcur_storage_mode);
}