/*********************************************************************************
  *Copyright(C),2015-2020,
  *TUYA
  *www.tuya.comm

  *FileName: tuya_ipc_dp_handler.c
  *
  * File Description：
  * 1. DP Point Setting and Acquisition Function API
  *
  * Developer work：
  * 1. Local configuration acquisition and update.
  * 2. Set local IPC attributes, such as picture flip, time watermarking, etc.
  *    If the function is not supported, leave the function blank.
  *
**********************************************************************************/
#include "tuya_iot_config.h"
#include "tuya_ipc_ptz.h"
#include "tuya_ipc_stream_storage.h"
#include "ty_dp_define.h"
#include "ty_sdk_common.h"
#include "utilities/uni_log.h"

#include "tkl_wifi.h"
#include <fcntl.h>
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

extern CHAR_T s_ipc_storage[64];

STATIC VOID __tuya_json_write(CHAR_T* buffer)
{
    CHAR_T config_path[256] = { 0 };
    snprintf(config_path, 256, "%s/tuya_dp_cfg.json", s_ipc_storage);

    FILE* fp = fopen(config_path, "w");
    fwrite(buffer, strlen(buffer), 1, fp);
    fclose(fp);
}

VOID TUYA_IPC_INIT_DP_CONFIG()
{
    CHAR_T config_path[256] = { 0 };
    INT_T fd = -1;
    INT_T sleep_mode = 0;
    INT_T light_switch = 0;
    INT_T flip_switch = 0;
    INT_T watermark_switch = 0;
    INT_T wdr_switch = 0;
    INT_T alarm_switch = 0;
    INT_T alarm_zone_switch = 0;
    INT_T sd_record_mode = 0;
    INT_T sd_record_switch = 0;
    INT_T blub_switch = 0;
    INT_T ap_switch = 0;
    CHAR_T night_mode[8] = { 0 };
    CHAR_T alarm_sen[8] = { 0 };
    CHAR_T ap_ssid[32] = { 0 };
    CHAR_T ap_passwd[32] = { 0 };

    snprintf(config_path, 256, "%s/tuya_dp_cfg.json", s_ipc_storage);
    if (0 == access(config_path, F_OK)) {
        return;
    } else {
        fd = open(config_path, O_RDWR | O_CREAT, 0666);
        if (fd < 0) {
            PR_ERR("create %s failed \n", config_path);
            return;
        }
        ty_cJSON* cfg_json = ty_cJSON_CreateObject();
        ty_cJSON_AddItemToObject(cfg_json, "tuya_sleep_mode", ty_cJSON_CreateNumber(sleep_mode));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_light_onoff", ty_cJSON_CreateNumber(light_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_flip_onoff", ty_cJSON_CreateNumber(flip_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_watermark_onoff", ty_cJSON_CreateNumber(watermark_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_wdr_onoff", ty_cJSON_CreateNumber(wdr_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_alarm_on_off", ty_cJSON_CreateNumber(alarm_switch));
        ty_cJSON_AddItemToObject(cfg_json, "alarm_zone_on_off", ty_cJSON_CreateNumber(alarm_zone_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_sd_record_mode", ty_cJSON_CreateNumber(sd_record_mode));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_sd_record_on_off", ty_cJSON_CreateNumber(sd_record_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_blub_on_off", ty_cJSON_CreateNumber(blub_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_ap_on_off", ty_cJSON_CreateNumber(ap_switch));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_night_mode", ty_cJSON_CreateString(night_mode));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_alarm_sen", ty_cJSON_CreateString(alarm_sen));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_ap_ssid", ty_cJSON_CreateString(ap_ssid));
        ty_cJSON_AddItemToObject(cfg_json, "tuya_ap_passwd", ty_cJSON_CreateString(ap_passwd));

        CHAR_T* buffer = ty_cJSON_Print(cfg_json);
        __tuya_json_write(buffer);
        ty_cJSON_free(buffer);
        ty_cJSON_Delete(cfg_json);
    }
}

STATIC ty_cJSON* __tuya_json_parse()
{
    CHAR_T buffer[4096] = { 0 };
    INT_T read_len = sizeof(buffer);
    INT_T fd = -1;
    CHAR_T config_path[256] = { 0 };

    snprintf(config_path, 256, "%s/tuya_dp_cfg.json", s_ipc_storage);
    fd = open(config_path, O_RDWR);
    if (fd < 0) {
        PR_ERR("create %s failed \n", config_path);
        return NULL;
    }

    while (read_len == read(fd, buffer, sizeof(buffer)))
        ;
    close(fd);

    ty_cJSON* cfg_json = ty_cJSON_Parse(buffer);
    return cfg_json;
}

/* Setting and Getting Integer Local Configuration.
The reference code here is written directly to tmp, requiring developers to replace the key.*/
STATIC VOID __tuya_app_write_INT(CHAR_T* key, INT_T value)
{
    ty_cJSON* cfg_json = __tuya_json_parse();
    if (NULL == cfg_json) {
        PR_ERR("load configure failed\n");
        return;
    }
    ty_cJSON_ReplaceItemInObject(cfg_json, key, ty_cJSON_CreateNumber(value));
    CHAR_T* json_tmp = ty_cJSON_Print(cfg_json);
    __tuya_json_write(json_tmp);
    ty_cJSON_free(json_tmp);
    ty_cJSON_Delete(cfg_json);
}

STATIC INT_T __tuya_app_read_INT(CHAR_T* key)
{
    int tmp_val = 0;
    ty_cJSON* cfg_json = __tuya_json_parse();
    if (NULL == cfg_json) {
        PR_ERR("load configure failed\n");
        return OPRT_CJSON_PARSE_ERR;
    }

    ty_cJSON* val = ty_cJSON_GetObjectItem(cfg_json, key);
    if (val) {
        tmp_val = val->valueint;
    }
    ty_cJSON_Delete(cfg_json);
    return tmp_val;
}

/* Setting and Getting String Local Configuration.
The reference code here is written directly to tmp, requiring developers to replace the key.*/
STATIC VOID __tuya_app_write_STR(CHAR_T* key, CHAR_T* value)
{
    ty_cJSON* cfg_json = __tuya_json_parse();
    if (NULL == cfg_json) {
        PR_ERR("load configure failed\n");
        return;
    }
    ty_cJSON_ReplaceItemInObject(cfg_json, key, ty_cJSON_CreateString(value));
    CHAR_T* json_tmp = ty_cJSON_Print(cfg_json);
    __tuya_json_write(json_tmp);
    ty_cJSON_free(json_tmp);
    ty_cJSON_Delete(cfg_json);
}

STATIC INT_T __tuya_app_read_STR(CHAR_T* key, CHAR_T* value, INT_T value_size)
{
    memset(value, 0, value_size);
    ty_cJSON* cfg_json = __tuya_json_parse();
    if (NULL == cfg_json) {
        PR_ERR("load configure failed\n");
        return OPRT_CJSON_PARSE_ERR;
    }
    ty_cJSON* val = ty_cJSON_GetObjectItem(cfg_json, key);
    if (val) {
        strncpy(value, val->valuestring, value_size);
    }
    ty_cJSON_Delete(cfg_json);
    return 0;
}

BOOL_T IPC_APP_get_alarm_function_onoff(VOID)
{
    BOOL_T alarm_on_off = __tuya_app_read_INT("tuya_alarm_on_off");
    PR_INFO("curr alarm_on_off:%d \r\n", alarm_on_off);
    return alarm_on_off;
}
INT_T IPC_APP_get_sd_status(VOID)
{
    INT_T sd_status = 1;
    /* SD card status, VALUE type, 1-normal, 2-anomaly, 3-insufficient space, 4-formatting, 5-no SD card */
    /* Developer needs to return local SD card status */
    // TODO
    sd_status = 1;

    PR_INFO("curr sd_status:%d \r\n", sd_status);
    return sd_status;
}

//------------------------------------------

VOID IPC_APP_get_sd_storage(UINT_T* p_total, UINT_T* p_used, UINT_T* p_empty)
{ // unit is kb
    // TODO
    /* Developer needs to return local SD card status */
    *p_total = 128 * 1000 * 1000;
    *p_used = 32 * 1000 * 1000;
    *p_empty = *p_total - *p_used;

    PR_INFO("curr sd total:%u used:%u empty:%u \r\n", *p_total, *p_used, *p_empty);
}
//------------------------------------------

//------------------------------------------

BOOL_T IPC_APP_unmount_sd_card(VOID)
{
    BOOL_T umount_ok = TRUE;

    // TODO
    /* unmount sdcard */

    PR_INFO("unmount result:%d \r\n", umount_ok);
    return umount_ok;
}

//------------------------------------------

/* -2000: SD card is being formatted, -2001: SD card formatting is abnormal, -2002: No SD card,
   -2003: SD card error. Positive number is formatting progress */
STATIC INT_T s_sd_format_progress = 0;
void* thread_sd_format(void* arg)
{
    /* First notify to app, progress 0% */
    s_sd_format_progress = 0;
    IPC_APP_report_sd_format_status(s_sd_format_progress);
    tal_system_sleep(1000);

    /* Stop local SD card recording and playback, progress 10%*/
    s_sd_format_progress = 10;
    IPC_APP_report_sd_format_status(s_sd_format_progress);
    // tuya_ipc_ss_set_write_mode(SS_WRITE_MODE_NONE);
    tuya_ipc_ss_pb_stop_all();
    tal_system_sleep(1000);

    /* Delete the media files in the SD card, the progress is 30% */
    s_sd_format_progress = 30;
    IPC_APP_report_sd_format_status(s_sd_format_progress);
    // tuya_ipc_ss_delete_all_files();
    tal_system_sleep(1000);

    /* Perform SD card formatting operation */
    // tuya_ipc_sd_format();

    s_sd_format_progress = 80;
    IPC_APP_report_sd_format_status(s_sd_format_progress);
    // TODO
    //  tuya_ipc_ss_set_write_mode(SS_WRITE_MODE_ALL);
    //    IPC_APP_set_sd_record_onoff( IPC_APP_get_sd_record_onoff());

    tal_system_sleep(1000);
    IPC_APP_report_sd_storage();
    /* progress 100% */
    s_sd_format_progress = 100;
    IPC_APP_report_sd_format_status(s_sd_format_progress);

    pthread_exit(0);
}

VOID IPC_APP_format_sd_card(VOID)
{
    PR_INFO("start to format sd_card \r\n");
    /* SD card formatting.
     * The SDK has already completed the writing of some of the code,
     and the developer only needs to implement the formatting operation. */

    pthread_t sd_format_thread;
    pthread_create(&sd_format_thread, NULL, thread_sd_format, NULL);
    pthread_detach(sd_format_thread);
}

INT_T IPC_APP_get_sd_format_status(VOID)
{
    return s_sd_format_progress;
}

VOID IPC_APP_set_sd_record_mode(UINT_T sd_record_mode)
{
    PR_INFO("set sd_record_mode:%d \r\n", sd_record_mode);
    __tuya_app_write_INT("tuya_sd_record_mode", sd_record_mode);
    
    if(0 == sd_record_mode)
    {
         tuya_ipc_ss_set_write_mode(SS_WRITE_MODE_EVENT);
    }
    else if(1 == sd_record_mode)
    {
         tuya_ipc_ss_set_write_mode(SS_WRITE_MODE_ALL);
    }
    else
    {
        PR_INFO("Error, should not happen\r\n");
        tuya_ipc_ss_set_write_mode(SS_WRITE_MODE_ALL);
    }
}

UINT_T IPC_APP_get_sd_record_mode(VOID)
{
    BOOL_T sd_record_mode = __tuya_app_read_INT("tuya_sd_record_mode");
    PR_TRACE("curr sd_record_mode:%d \r\n", sd_record_mode);
    return sd_record_mode;
}

//------------------------------------------

#ifdef TUYA_DP_ELECTRICITY
INT_T IPC_APP_get_battery_percent(VOID)
{
    // TODO
    /* battery power percentage VALUE type,[0-100] */

    return 80;
}
#endif

#ifdef TUYA_DP_POWERMODE
CHAR_T* IPC_APP_get_power_mode(VOID)
{
    // TODO
    /* Power supply mode, ENUM type,
    "0" is the battery power supply state, "1" is the plug-in power supply state (or battery charging state) */

    return "1";
}
#endif
